#include <stdio.h>
#include <stdlib.h> /* atoi */
#include <windows.h>
#include <winsock.h>

#include "as.h"
#include "resource.h"

#ifdef UNICODE
#define STRICMP wcsicmp
#define ITOA(c, sz, b) (itoa(sizeof(szA), szA, b), mbstowcs(sz, szA, b), sz)
#else
#define STRICMP stricmp
#define ITOA itoa
#endif	// UNICODE


HANDLE	    hInst;			// our instance/module handle
HWND	    hwndMain;			// our main window handle
#ifdef UNICODE
CHAR	    szA[20];			// static buffer for UNICODE conversion
TCHAR	    szTitle[] = TEXT("ASPP (U)");
#else
TCHAR	    szTitle[] = TEXT("ASPP");
#endif


static SOCKET	FdCmd  = INVALID_SOCKET;
static SOCKET	FdData = INVALID_SOCKET;

static int	    ConnectCN2100();
static void	    Show();
DWORD FAR PASCAL    Rx( LPSTR lpData );
extern void	    ConfigInit();


int WINAPI WinMain(
HINSTANCE hInstance,
HINSTANCE hPrevInstance,
LPSTR lpCmdLine,
INT nCmdShow)
{
    MSG msg;
    WSADATA wsaData;

    UNREFERENCED_PARAMETER( lpCmdLine );

    if (!hPrevInstance) 		 /* Other instances of app running? */
	if (!InitApplication(hInstance)) /* Initialize shared things	    */
	    return (FALSE);		 /* Exits if unable to initialize   */

    if (WSAStartup(0x202,&wsaData) == SOCKET_ERROR) {
	WSACleanup();
	UnregisterClass(szTitle, hInstance);
	return(FALSE);
    }

    /*
    *	Perform initializations that apply to a specific instance
    */
    if (!InitInstance(hInstance, nCmdShow))
	return (FALSE);

    while (GetMessage(&msg, 0, 0, 0)) {
	TranslateMessage(&msg);
	DispatchMessage(&msg);
    }

    WSACleanup();
    DestroyWindow(hwndMain);
    UnregisterClass(szTitle, hInstance);
    return(FALSE);
}

/****************************************************************************
*
*    FUNCTION: InitApplication(HANDLE)
*
*    PURPOSE: Initializes window data and registers window class
*
*\***************************************************************************/

BOOL InitApplication(HANDLE hInstance)	     /* current instance	     */
{
    WNDCLASS  wc;

/*  wc.style = CS_HREDRAW | CS_VREDRAW; */
    wc.style = 0;
    wc.lpfnWndProc = MainWndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = NULL;
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = NULL;
    wc.lpszMenuName =  "ExampleMenu";
    wc.lpszClassName = szTitle;

    return (RegisterClass(&wc));

}


/****************************************************************************\
*
*    FUNCTION:	InitInstance(HANDLE, int)
*
*    PURPOSE:  Saves instance handle and creates main window
*
*\***************************************************************************/

BOOL InitInstance(
    HANDLE	    hInstance,		/* Current instance identifier.       */
    int 	    nCmdShow)		/* Param for first ShowWindow() call. */
{
	HWND	hwnd;

    /* Save the instance handle in static variable, which will be used in  */
    /* many subsequence calls from this application to Windows. 	   */

    hInst = hInstance;

    /* Create a main window for this application instance.  */

    hwnd = CreateWindow(
	szTitle,
	szTitle,
	WS_OVERLAPPEDWINDOW,
	CW_USEDEFAULT,	CW_USEDEFAULT,
	CW_USEDEFAULT,
	CW_USEDEFAULT,
	NULL,
	NULL,
	hInstance,
	NULL
    );

    /* If window could not be created, return "failure" */

    if (!hwnd) {
	return(FALSE);
    }

    ShowWindow(hwnd, nCmdShow);
    UpdateWindow(hwnd);
    return (TRUE);		 /* Returns the value from PostQuitMessage */
}


static char CrLf[2] = "\r\n";

#define BUFFERLEN    80 * 24

static CHAR    Buffer[BUFFERLEN];
static INT     Len     = 0;
static INT     CxChar,CyChar;

LONG  APIENTRY MainWndProc(
HWND hwnd,
UINT message,
WPARAM wParam,
LONG lParam)
{
	PAINTSTRUCT ps;
	RECT	    rc;
	HDC	    hdc;
	HMENU	    hMenu ;
	TEXTMETRIC  tm;
	HANDLE	    rt;
	DWORD	    dwThreadID;

	switch (message) {
	case WM_CREATE:

	    hdc = GetDC(hwnd);
	    GetTextMetrics(hdc,&tm);
	    CxChar = tm.tmAveCharWidth;
	    CyChar = tm.tmHeight;
	    ReleaseDC(hwnd,hdc);

	    hwndMain = hwnd;
	    hMenu = GetMenu( hwnd ) ;
	    EnableMenuItem( hMenu, IDM_CONNECT, MF_ENABLED | MF_BYCOMMAND ) ;
	    EnableMenuItem( hMenu, IDM_DISCONNECT,
		   MF_GRAYED | MF_DISABLED | MF_BYCOMMAND ) ;

	    ConfigInit();
	    break;
	case WM_COMMAND:
	    switch (wParam) {
		case IDM_CONFIG 	:
		     DialogBox(hInst,
			"ConfigBox",
			hwnd,
			Config);
		    break;
		case IDM_CONNECT	:
		    if (ConnectCN2100()) {
			MessageBox(hwnd,"CN2100 Connect fail!",
				    NULL,MB_OK | MB_ICONHAND);
			break;
		    }
		    rt = CreateThread( (LPSECURITY_ATTRIBUTES) NULL,
			    0,
			    (LPTHREAD_START_ROUTINE) Rx,
			    (LPVOID) NULL,
			    0, &dwThreadID );
		    if (rt == NULL) {
			MessageBox(hwnd,"CREATE Rx THread fail!",
				    NULL,MB_OK | MB_ICONHAND);
			closesocket(FdCmd);
			closesocket(FdData);
			FdCmd = INVALID_SOCKET;
			FdData = INVALID_SOCKET;
			break;
		    }
		    hMenu = GetMenu( hwnd ) ;
		    EnableMenuItem( hMenu, IDM_DISCONNECT,
			MF_ENABLED | MF_BYCOMMAND ) ;
		    EnableMenuItem( hMenu, IDM_CONNECT,
			MF_GRAYED | MF_DISABLED | MF_BYCOMMAND ) ;
		    break;
		case IDM_DISCONNECT	:
		    if (FdCmd != INVALID_SOCKET)
			closesocket(FdCmd);
		    if (FdData != INVALID_SOCKET)
			closesocket(FdData);
		    FdCmd = INVALID_SOCKET;
		    FdData = INVALID_SOCKET;
		    hMenu = GetMenu( hwnd ) ;
		    EnableMenuItem( hMenu, IDM_CONNECT,
			MF_ENABLED | MF_BYCOMMAND ) ;
		    EnableMenuItem( hMenu, IDM_DISCONNECT,
			MF_GRAYED | MF_DISABLED | MF_BYCOMMAND ) ;
		    break;
		case IDM_ABOUT		:
		     DialogBox(hInst,
			"AboutBox",
			hwnd,
			About);
		    break;
		default:		/* Lets Windows process it	 */
		    return (DefWindowProc(hwnd, message, wParam, lParam));
	    }
	    break;
	case WM_CHAR:
	    if (FdData != INVALID_SOCKET) {
		if ((wParam == '\r') || (wParam == '\n'))
		    send(FdData, CrLf, 2, 0);
		else
		    send(FdData, (char *)&wParam, 1, 0);
	    }
	    break;
	case WM_PAINT:
	    hdc = GetDC(hwnd);
	    BeginPaint(hwnd, &ps);
	    GetClientRect(hwnd,&rc);
	    SetBkMode(ps.hdc, TRANSPARENT);
	    SetTextColor(ps.hdc, 0x00FFFFFF);	// white text
	    FillRect(ps.hdc, &rc, GetStockObject(WHITE_BRUSH));
	    EndPaint(hwnd, &ps);
	    Show();
	    ReleaseDC(hwnd,hdc);
	    break;
	case WM_CLOSE:
	    if (FdCmd != INVALID_SOCKET)
		closesocket(FdCmd);
	    if (FdData != INVALID_SOCKET)
		closesocket(FdData);
	    Sleep(200); 			/* Wait for Rx thread exit */
	    PostQuitMessage(0);
	    break;

	default:
	    return (DefWindowProc(hwnd, message, wParam, lParam));
	}
	return(0);
}


extern	unsigned long	Ip;
extern	int		Port;
extern	int		Baud;
extern	int		Bit;
extern	int		Stop;
extern	int		Parity;
extern	int		Flow;


static int     ConnectCN2100()
{
	int mode,cts,rts,txxon,rxxon;


	if((FdCmd = sio_copen(Ip,Port))==INVALID_SOCKET)
	    return(-1);
	if((FdData = sio_dopen(Ip,Port))==INVALID_SOCKET) {
	    closesocket(FdCmd);
	    FdCmd = INVALID_SOCKET;
	    return(-1);
	}
	switch (Bit) {
	    case 0 :
		mode = D_IOCTL_BIT5;
		break;
	    case 1 :
		mode = D_IOCTL_BIT6;
		break;
	    case 2 :
		mode = D_IOCTL_BIT7;
		break;
	    case 3 :
		mode = D_IOCTL_BIT8;
		break;
	    default:
		mode = D_IOCTL_BIT8;
		break;
	}
	switch (Stop) {
	    case 0 :
		mode |= D_IOCTL_STOP1;
		break;
	    case 1 :
		mode |= D_IOCTL_STOP2;
		break;
	    default:
		mode |= D_IOCTL_STOP1;
		break;
	}
	switch (Parity) {
	    case 0 :
		mode |= D_IOCTL_NONE;
		break;
	    case 1 :
		mode |= D_IOCTL_EVEN;
		break;
	    case 2 :
		mode |= D_IOCTL_ODD;
		break;
	    default:
		mode |= D_IOCTL_NONE;
		break;
	}
	if (sio_ioctl(FdCmd,Baud,mode)) {
	    closesocket(FdCmd);
	    FdCmd = INVALID_SOCKET;
	    closesocket(FdData);
	    FdData = INVALID_SOCKET;
	    return(-1);
	}
	switch (Flow) {
	    case 0 :
		cts = rts = 0;
		txxon = rxxon = 0;
		break;
	    case 1 :
		cts = rts = 1;
		txxon = rxxon = 0;
		break;
	    case 2 :
		cts = rts = 0;
		txxon = rxxon = 1;
		break;
	    default :
		cts = rts = 0;
		txxon = rxxon = 0;
		break;
	}
	if (sio_flowctrl(FdCmd,cts,rts,txxon,rxxon)) {
	    closesocket(FdCmd);
	    FdCmd = INVALID_SOCKET;
	    closesocket(FdData);
	    FdData = INVALID_SOCKET;
	    return(-1);
	}
	send(FdData, "CN2100 ASPP Example program for NT/95.  \r\n", 42, 0);
	return(0);
}

DWORD FAR PASCAL    Rx( LPSTR lpData )
{
	char	    buf[80];
	int	    len;

	InvalidateRect(hwndMain,NULL,TRUE);
	Len = 0;
	for (;;) {
	    if (FdData == INVALID_SOCKET)
		break;
	    len=recv(FdData, buf, sizeof(buf), 0);
	    if (len > 0) {
		if ((len + Len) > BUFFERLEN) {
		    Len = 0;
		    InvalidateRect(hwndMain,NULL,TRUE);
		}
		memcpy(&Buffer[Len],buf,len);
		Len += len;
		Show();
	    } else
		Sleep(100);			/* sleep 100 ms */
	}
	ExitThread(0);
	return(0);
}

static void    Show()
{
	int	cx,cy,i;
	HDC	hdc;

	hdc = GetDC(hwndMain);
	cx = cy = 0;
	for (i=0;i<Len;i++) {
	    if (Buffer[i] == '\r')
		cx = 0;
	    else if (Buffer[i] == '\n')
		cy += CyChar;
	    else {
		TextOut(hdc,cx,cy,&Buffer[i],1);
		cx += CxChar;
	    }
	}
	ReleaseDC(hwndMain,hdc);
}
